/*
 * Copyright (c) 2021, Idan Horowitz <idan.horowitz@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <AK/TestSuite.h>

#include <AK/Random.h>
#include <AK/RedBlackTree.h>

TEST_CASE(construct)
{
    RedBlackTree<int, int> empty;
    EXPECT(empty.is_empty());
    EXPECT(empty.size() == 0);
}

TEST_CASE(ints)
{
    RedBlackTree<int, int> ints;
    ints.insert(1, 10);
    ints.insert(3, 20);
    ints.insert(2, 30);
    EXPECT_EQ(ints.size(), 3u);
    EXPECT_EQ(*ints.find(3), 20);
    EXPECT_EQ(*ints.find(2), 30);
    EXPECT_EQ(*ints.find(1), 10);
    EXPECT(!ints.remove(4));
    EXPECT(ints.remove(2));
    EXPECT(ints.remove(1));
    EXPECT(ints.remove(3));
    EXPECT_EQ(ints.size(), 0u);
}

TEST_CASE(largest_smaller_than)
{
    RedBlackTree<int, int> ints;
    ints.insert(1, 10);
    ints.insert(11, 20);
    ints.insert(21, 30);
    EXPECT_EQ(ints.size(), 3u);
    EXPECT_EQ(*ints.find_largest_not_above(3), 10);
    EXPECT_EQ(*ints.find_largest_not_above(17), 20);
    EXPECT_EQ(*ints.find_largest_not_above(22), 30);
    EXPECT_EQ(ints.find_largest_not_above(-5), nullptr);
}

TEST_CASE(key_ordered_iteration)
{
    constexpr auto amount = 10000;
    RedBlackTree<int, size_t> test;
    Array<int, amount> keys {};

    // generate random key order
    for (int i = 0; i < amount; i++) {
        keys[i] = i;
    }
    for (size_t i = 0; i < amount; i++) {
        swap(keys[i], keys[get_random<size_t>() % amount]);
    }

    // insert random keys
    for (size_t i = 0; i < amount; i++) {
        test.insert(keys[i], keys[i]);
    }

    // check key-ordered iteration
    size_t index = 0;
    for (auto& value : test) {
        EXPECT(value == index++);
    }

    // ensure we can remove all of them (aka, tree structure is not destroyed somehow)
    for (size_t i = 0; i < amount; i++) {
        EXPECT(test.remove(i));
    }
}

TEST_CASE(clear)
{
    RedBlackTree<size_t, size_t> test;
    for (size_t i = 0; i < 1000; i++) {
        test.insert(i, i);
    }
    test.clear();
    EXPECT_EQ(test.size(), 0u);
}

TEST_MAIN(RedBlackTree)
